#######################
## ABI functions...
#######################
require(maDB)

getAssayNames <- function(file,check.names=FALSE){
	head <- read.table(file,sep="\t",as.is=TRUE,header=TRUE,nrows=2,check.names=check.names)
	columns <- colnames(head)
	assays <- columns[grep(columns,pattern="_ASSAY_NAME")]
#	ret <- replaceAll(assays,old="_ASSAY_NAME",new="")
	ret <- gsub("_ASSAY_NAME","",assays)
	ret
}


readABIData <- function(files,assays=NULL,signal="SIGNAL",v=TRUE,flagabove=100,check.names=FALSE){
	if(length(files)==1){
		if(!is.null(assays)){
			if(!class(assays)=="list"){
				assays <- list(assays)
			}
		}
		else{
			assays <- list(assays)
		}
	}
	for(i in 1:length(files)){
		if(i==1){
			eset <- .readABI(files[i],assays=assays[[i]],signal=signal,v=v,flagabove=flagabove,check.names=check.names)
		}
		else{
			dummy <- .readABI(files[i],assays=assays[[i]],signal=signal,v=v,flagabove=flagabove,check.names=check.names)
			eset@exprs <- cbind(eset@exprs,dummy@exprs[rownames(eset@exprs),])
			eset@weights <- cbind(getWeights(eset),getWeights(dummy)[rownames(eset@exprs),])
		}
	}
	eset
}

flag <- function(x,level){
	if(x>=level){
		ret <- 0
	}
	else{
		ret <- 1
	}
	ret
}

.readABI <- function(file,assays=NULL,signal="SIGNAL",v=TRUE,flagabove=100,check.names=FALSE){
	dummy <- read.table(file,sep="\t",as.is=TRUE,header=TRUE,check.names=check.names)
	if(is.null(assays)){
		if(v){
			cat("Reading all assays from the file ",file,"\n")
		}
		assays <- getAssayNames(file,check.names=check.names)
	}
	eset <- new("EexprSet")
#	cat(assays,"\n")
#	cat(paste(assays,"_",signal,sep=""))
	dummy2 <- dummy[,paste(assays,"_",signal,sep="")]
	dummy2 <- apply(dummy2,c(1,2),gsub,pattern=",",replacement="")
	dummy2 <- apply(dummy2,c(1,2),as.numeric)
	eset@exprs <- as.matrix(dummy2)
	rm(dummy2)
	colnames(eset@exprs) <- assays
	rownames(eset@exprs) <- dummy[,"GENE_ID"]
	c.names <- "GENE_ID"
	if(sum(colnames(dummy)=="PROBE_ID")>0){
		c.names <- c(c.names,"PROBE_ID")
		#cat(length(dummy[,"PROBE_ID"]),length(unique(dummy[,"PROBE_ID"])),"\n")
	}
	eset@genes <- as.matrix(dummy[,c.names])
	colnames(eset@genes) <- c.names
	
	##
	# flagging
	if(!is.null(flagabove)){
		weights <- dummy[,paste(assays,"_FLAGS",sep="")]
		weights <- apply(weights,c(1,2),gsub,pattern=",",replacement="")
		weights <- apply(weights,c(1,2),as.numeric)
		rownames(weights) <- dummy[,"GENE_ID"]
		colnames(weights) <- assays
		weights <- apply(weights,c(1,2),flag,level=flagabove)
		eset@weights <- as.matrix(weights)
	}
	eset
}

